#!/usr/bin/env python
"""
_AddToFilesByIDs_

Oracle implementation of AddFileToFilesetByIDs
"""

from WMCore.WMBS.MySQL.Files.AddToFilesetByIDs import AddToFilesetByIDs as AddFileToFilesetByIDsMySQL

class AddToFilesetByIDs(AddFileToFilesetByIDsMySQL):
    pass
