"""
__UpdatePriority__

Oracle implementation of UpdatePriority

Created on Apr 16, 2013

@author: dballest
"""

from WMCore.WMBS.MySQL.Workflow.UpdatePriority import UpdatePriority as MySQLUpdatePriority

class UpdatePriority(MySQLUpdatePriority):
    pass
